<?php

class Related extends DataObjectDecorator {
	static $onWriteRelated = null; // Array which remembers related pages before write

	public function extraStatics() {
		return array(
			'db' => array (
				'RelatedCaption' => 'Text', // Used in DOM as no function calls allowed from SiteTree based classes
			),
			'belongs_many_many' => array(
				'RelatedToMe' => __CLASS__ // One Item
			),
			'many_many' => array(
				'Related' => __CLASS__		// Multiple items
			)
		);
	}

	public function parentClass() {
		//return self::get_classname_without_arguments;
	}

	/**
	 * Overwrite to also check for method on the original data object
	 *
	 * @param string $method
	 * @return bool
	 */
	function hasMethod($method) {
		return method_exists($this, $method) || isset(self::$extra_methods[$this->class][strtolower($method)]);
	}


	public function updateCMSFields(&$fields) {
//		Debug::Show($this->owner->ClassName);
		$manager = new ManyManyDataObjectManager($this->owner, "Related", "Page", array('RelatedCaption' => __CLASS__, 'Title' => 'Title'), "getCMSFields_forPopup", "", "RelatedCaption ASC");
		$manager->setPermissions(array());
//		$manager = new ManyManyComplexTableField($this, "Related", "Page", array('RelatedCaption' => __CLASS__, 'Title' => 'Title'), "getCMSFields_forPopup", "", "RelatedCaption ASC");

		$fields->addFieldToTab("Root.Related", $manager);
	}

	public function getCMSFields_forPopup() {
		$fields = new FieldSet(
			new TextField("Title", "Title")
		);
		return $fields;
	}


	/**
	 * Return a MenuTitle based breadcrumb trail to this page. Excludes "hidden" pages
	 * (with ShowInMenus=0).
	 *
	 * @param int $maxDepth The maximum depth to traverse.
	 * @param boolean $unlinked Do not make page names links
	 * @param string $stopAtPageType ClassName of a page to stop the upwards traversal.
	 * @param boolean $showHidden Include pages marked with the attribute ShowInMenus = 0
	 * @return string The breadcrumb trail.
	 */
	public function MenuTitleBreadcrumbs($maxDepth = 20, $unlinked = false, $stopAtPageType = false, $showHidden = false) {
		$page = $this->owner;
		$parts = array();
		$i = 0;
		while(
			$page
 			&& (!$maxDepth || sizeof($parts) < $maxDepth)
 			&& (!$stopAtPageType || $page->ClassName != $stopAtPageType)
 		) {
			if($showHidden || $page->ShowInMenus || ($page->ID == $this->owner->ID)) {
				if($page->URLSegment == 'home') $hasHome = true;
				if(($page->ID == $this->owner->ID) || $unlinked) {
				 	$parts[] = Convert::raw2xml($page->MenuTitle);
				} else {
					$parts[] = ("<a href=\"" . $page->Link() . "\">" . Convert::raw2xml($page->MenuTitle) . "</a>");
				}
			}
			$page = $page->Parent;
		}

		return implode(SiteTree::$breadcrumbs_delimiter, array_reverse($parts));
	}

	public function onBeforeWrite() {
		parent::onBeforeWrite();
		$this->owner->RelatedCaption = str_replace(SiteTree::$breadcrumbs_delimiter, " > ", $this->owner->MenuTitleBreadcrumbs(20, true));

		if (is_null($this->owner->onWriteRelated)) {
			$this->owner->onWriteRelated = new DataObjectSet();
			foreach($this->owner->Related() as $page) {
				$this->owner->onWriteRelated->push($page->ID);
			}
		}
	}

	/**
	 * Ensure relation including deletions are also written to other side of relation
	 */
	public function onAfterWrite() {
		parent::onAfterWrite();

		foreach($this->owner->onWriteRelated as $ID) {
			if (!$this->owner->Related()->containsIDs(array($ID))) {
				DB::query("DELETE FROM " . __CLASS__ . "_Related
									 WHERE (" . __CLASS__ . "ID = {$ID} AND ChildID = {$this->owner->ID})
									 	  OR (" . __CLASS__ . "ID = {$this->owner->ID} AND ChildID = {$ID})");
			}
		}

		$query = new SQLQuery(
			$select = "`" . __CLASS__ . "_Related`." . __CLASS__ . "ID, `" . __CLASS__ . "_Related`.ChildID",
			$from = "`" . __CLASS__ . "_Related`",
			$where = "`" . __CLASS__ . "_Related`." . __CLASS__ . "ID = {$this->owner->ID} OR `" . __CLASS__ . "_Related`.ChildID = {$this->owner->ID}"
		);

		$result = $query->execute();
		$pairs = array();
		foreach($result as $row) {
			$pairs[$row[__CLASS__ . "ID"]][$row["ChildID"]] = true;
		}
		$insert = array();
		foreach($this->owner->Related() as $page) {
			if (!isset($pairs[$page->ID][$this->owner->ID])) {
				$insert[] = $page->ID;
			}
		}

		if (count($insert) > 0) {
			$values = "";
			foreach($insert as $value) {
				if ($values != "") $values .= ", ";
				$values .= "(" . $value . ", '{$this->owner->ID}')";
			}
			DB::query("INSERT INTO " . __CLASS__ . "_Related (`" . __CLASS__ . "ID`, `ChildID`)
								 VALUES " . $values);
		}
	}
}

